@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header$
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.h,v 1.23 2010/08/26 09:23:06 kitagawa Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -19,7 +19,8 @@
  * suitability of this software for any purpose.  It is provided "as
  * is" without express or implied warranty.
  *
- * pool_proto_modules.h.: header file for pool_proto_modules.c and pool_process_qeury.c
+ * pool_proto_modules.h.: header file for pool_proto_modules.c, pool_proto2.c
+ * and pool_process_qeury.c
  *
  */
 
@@ -31,46 +32,15 @@
 #include "parser/pg_list.h"
 #include "parser/parsenodes.h"
 #include "pool_rewrite_query.h"
+#include "pool_session_context.h"
 
 #define SPECIFIED_ERROR 1
 #define POOL_ERROR_QUERY "send invalid query from pgpool to abort transaction"
 
-
-/* Prepared statement information */
-typedef struct {
-	char *portal_name; /* portal name*/
-	Node *stmt;        /* parse tree for prepared statement */
-	char *sql_string;  /* original SQL statement */
-	POOL_MEMORY_POOL *prepare_ctxt; /* memory context for parse tree */
-	int num_tsparams;
-} Portal;
-
-/*
- * prepared statement list
- */
-typedef struct {
-	int size;
-	int cnt;
-	Portal **portal_list;
-} PreparedStatementList;
-
-extern int force_replication;
-extern int replication_was_enabled;		/* replication mode was enabled */
-extern int master_slave_was_enabled;	/* master/slave mode was enabled */
-extern int internal_transaction_started;		/* to issue table lock command a transaction
-												   has been started internally */
-extern int in_progress;		/* indicates while doing something after receiving Query */
-extern int mismatch_ntuples;	/* number of updated tuples */
 extern char *copy_table;  /* copy table name */
 extern char *copy_schema;  /* copy table name */
 extern char copy_delimiter; /* copy delimiter char */
 extern char *copy_null; /* copy null string */
-extern void (*pending_function)(PreparedStatementList *p, Portal *portal);
-extern Portal *pending_prepared_portal;
-extern Portal *unnamed_statement;
-extern Portal *unnamed_portal;
-extern int select_in_transaction; /* non 0 if select query is in transaction */
-extern int execute_select; /* non 0 if select query is in transaction */
 
 /* non 0 if "BEGIN" query with extended query protocol received */
 extern int receive_extended_begin;
@@ -80,28 +50,54 @@ extern int is_select_for_update; /* also for SELECT ... INTO */
 extern bool is_parallel_table;
 extern char *parsed_query;
 
-extern PreparedStatementList prepared_list; /* prepared statement name list */
-
 /*
  * modules defined in pool_proto_modules.c
  */
-extern POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend, 
-										POOL_CONNECTION_POOL *backend);
-
 extern POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend, 
-						 POOL_CONNECTION_POOL *backend, char *query);
+							   POOL_CONNECTION_POOL *backend,
+							   int len, char *contents);
 
 extern POOL_STATUS Execute(POOL_CONNECTION *frontend, 
-						   POOL_CONNECTION_POOL *backend);
+						   POOL_CONNECTION_POOL *backend,
+						   int len, char *contents);
 
 extern POOL_STATUS Parse(POOL_CONNECTION *frontend,
-						 POOL_CONNECTION_POOL *backend);
+						 POOL_CONNECTION_POOL *backend,
+						 int len, char *contents);
+
+extern POOL_STATUS Bind(POOL_CONNECTION *frontend,
+						POOL_CONNECTION_POOL *backend,
+						int len, char *contents);
+
+extern POOL_STATUS Describe(POOL_CONNECTION *frontend,
+							POOL_CONNECTION_POOL *backend,
+							int len, char *contents);
+
+extern POOL_STATUS Close(POOL_CONNECTION *frontend,
+						 POOL_CONNECTION_POOL *backend,
+						 int len, char *contents);
+
+extern POOL_STATUS FunctionCall3(POOL_CONNECTION *frontend,
+								 POOL_CONNECTION_POOL *backend,
+								 int len, char *contents);
 
 extern POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready);
 
-extern POOL_STATUS CompleteCommandResponse(POOL_CONNECTION *frontend, 
-										   POOL_CONNECTION_POOL *backend);
+extern POOL_STATUS ParseComplete(POOL_CONNECTION *frontend,
+								 POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS BindComplete(POOL_CONNECTION *frontend,
+								POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS CloseComplete(POOL_CONNECTION *frontend,
+								 POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS CommandComplete(POOL_CONNECTION *frontend,
+								   POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS ErrorResponse3(POOL_CONNECTION *frontend,
+								  POOL_CONNECTION_POOL *backend);
 
 extern POOL_STATUS CopyInResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend);
@@ -112,32 +108,45 @@ extern POOL_STATUS CopyOutResponse(POOL_CONNECTION *frontend,
 extern POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 								POOL_CONNECTION_POOL *backend, int copyin);
 
-extern POOL_STATUS CursorResponse(POOL_CONNECTION *frontend, 
+extern POOL_STATUS FunctionCall(POOL_CONNECTION *frontend, 
+								POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend, 
+										   POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS ProcessBackendResponse(POOL_CONNECTION *frontend, 
+										  POOL_CONNECTION_POOL *backend,
+										  int *state, short *num_fields);
+
+/*
+ * modules defined in pool_proto2.c
+ */
+extern POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
+							POOL_CONNECTION_POOL *backend,
+							short num_fields);
+
+extern POOL_STATUS BinaryRow(POOL_CONNECTION *frontend,
+							 POOL_CONNECTION_POOL *backend,
+							 short num_fields);
+
+extern POOL_STATUS CompletedResponse(POOL_CONNECTION *frontend, 
+									 POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS CursorResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend);
 
 extern POOL_STATUS EmptyQueryResponse(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend);
 
-extern int RowDescription(POOL_CONNECTION *frontend, 
-						  POOL_CONNECTION_POOL *backend,
-						  short *result);
-
-extern POOL_STATUS AsciiRow(POOL_CONNECTION *frontend, 
-							POOL_CONNECTION_POOL *backend,
-							short num_fields);
-
-extern POOL_STATUS BinaryRow(POOL_CONNECTION *frontend, 
-							 POOL_CONNECTION_POOL *backend,
-							 short num_fields);
-
-extern POOL_STATUS FunctionCall(POOL_CONNECTION *frontend, 
-								POOL_CONNECTION_POOL *backend);
-
-extern POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend, 
+extern POOL_STATUS FunctionResultResponse(POOL_CONNECTION *frontend,
 										  POOL_CONNECTION_POOL *backend);
 
-extern POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend, 
-										   POOL_CONNECTION_POOL *backend);
+extern POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
+										POOL_CONNECTION_POOL *backend);
+
+extern int RowDescription(POOL_CONNECTION *frontend,
+						  POOL_CONNECTION_POOL *backend,
+						  short *result);
 
 
 extern POOL_STATUS wait_for_query_response(POOL_CONNECTION *frontend, POOL_CONNECTION *backend, char *string, int protoVersion);
@@ -147,15 +156,9 @@ extern int is_start_transaction_query(Node *node);
 extern int is_commit_query(Node *node);
 extern int is_strict_query(Node *node); /* returns non 0 if this is strict query */
 extern int load_balance_enabled(POOL_CONNECTION_POOL *backend, Node* node, char *sql);
-extern void start_load_balance(POOL_CONNECTION_POOL *backend);
-extern void end_load_balance(void);
 extern int need_insert_lock(POOL_CONNECTION_POOL *backend, char *query, Node *node);
-extern POOL_STATUS insert_lock(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, char *query, InsertStmt *node);
-extern void add_prepared_list(PreparedStatementList *p, Portal *portal);
-extern void add_unnamed_portal(PreparedStatementList *p, Portal *portal);
-extern void delete_all_prepared_list(PreparedStatementList *p, Portal *portal);
+extern POOL_STATUS insert_lock(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, char *query, InsertStmt *node, int lock_kind);
 extern char *parse_copy_data(char *buf, int len, char delimiter, int col_id);
-extern Portal *lookup_prepared_statement_by_portal(PreparedStatementList *p, const char *name);extern Portal *lookup_prepared_statement_by_statement(PreparedStatementList *p, const char *name);
 extern int check_copy_from_stdin(Node *node); /* returns non 0 if this is a COPY FROM STDIN */
 extern void query_ps_status(char *query, POOL_CONNECTION_POOL *backend);		/* show ps status */
 extern POOL_STATUS start_internal_transaction(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, Node *node);
@@ -166,23 +169,24 @@ extern int detect_active_sql_transaction_error(POOL_CONNECTION *backend, int maj
 extern int detect_query_cancel_error(POOL_CONNECTION *backend, int major);
 extern bool is_partition_table(POOL_CONNECTION_POOL *backend, Node *node);
 extern POOL_STATUS pool_discard_packet(POOL_CONNECTION_POOL *cp);
+extern void query_cache_register(char kind, POOL_CONNECTION *frontend, char *database, char *data, int data_len);
 
 extern int is_drop_database(Node *node);		/* returns non 0 if this is a DROP DATABASE command */
-extern void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
-extern Portal *create_portal(void);
-extern void del_prepared_list(PreparedStatementList *p, Portal *portal);
+extern void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
+extern void pools_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
+extern void processes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
+extern void nodes_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
+extern void version_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 
 extern POOL_STATUS send_simplequery_message(POOL_CONNECTION *backend, int len, char *string, int major);
 extern POOL_STATUS send_extended_protocol_message(POOL_CONNECTION_POOL *backend,
 												  int node_id, char *kind,
 												  int len, char *string);
 
-extern POOL_STATUS send_execute_message(POOL_CONNECTION_POOL *backend,
-										int node_id, int len, char *string);
-
 extern int synchronize(POOL_CONNECTION *cp);
 extern POOL_STATUS read_kind_from_backend(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, char *decided_kind);
 extern POOL_STATUS read_kind_from_one_backend(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, char *kind, int node);
 extern POOL_STATUS do_error_command(POOL_CONNECTION *backend, int major);
+extern POOL_STATUS raise_intentional_error_if_need(POOL_CONNECTION_POOL *backend);
 
 #endif
